#include "main.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../senior/source/parse.c"
#include "../senior/source/syn_cam.c"

#include "../senior/source/hemicube.c"

#if (defined(_HEMICUBE_H) || defined(_CUBICTETRA_H))
#include "../senior/source/prog_rad.c"
static ProgRad Radiosity;
#elif defined(_RAYCAST_H)
#include "../senior/include/ray_rad.h"
static RayRad Radiosity;
#else
#include "../senior/include/rad_eqn.h"
static RadEqnSolve Radiosity;
#endif

static const int MaxStep = 2000;
static const int MinPixel = 32;
static const int MaxPixel = 1024;
static Environ Environment;     //
static char *EntityDir;  // Entity directory
static Parse Parser;            // World file parser
static SynCamera Camera(320,240);
static BOOL RadCalcDone = FALSE;
//*****************************HEILOS INIT
static BOOL AbortFlag = FALSE;  // User-requested abort
static BOOL InCalc = FALSE;     // Calculations in progress
static BOOL EraseFlag = TRUE;   // Erase background flag
Environ *penv;
//******************************
// Display type
#define H_NONE      0   // None
#define H_WIRE      1   // Wireframe
#define H_BMAP      2   // Bitmap

/* a red,green,blue,alpha pixel */
typedef struct {unsigned char r, g, b, a;} Rgba_pixel;
// typedef struct
// {
//	const char *item_text;     /* item text label              */
//	FL_PUP_CB callback;        /* item callback routine        */
//	const char *shortcut;      /* shortcut for this item       */
//	unsigned int mode;         /* item mode                    */
//} FL_PUP_ENTRY;

//static Glxf_pane pane;	/* X-OpenGL integration */
MouseT mouse;		/* mouse status */
Pic *in,*in2;		/* loaded image */

BartParam BartlettParam[] =
{
  { 0.0000, 1, "None", "None" },
  { 0.5000, 3, "3 x 3", "3 x 3 Bartlett" },
  { 0.3333, 5, "5 x 5", "5 x 5 Bartlett" },
  { 0.2500, 7, "7 x 7", "7 x 7 Bartlett" }
};
PolyClip4 clipper;
typedef struct          // Bartlett filter element
{
  double x_offset;      // x-axis subpixel offset
  double y_offset;      // y-axis subpixel offset
  int alpha;            // Subpixel blend coefficient
  int beta;             // Pixel blend coefficient
  int weight;           // Subpixel weight
} Barlett;
BYTE __huge *pix;
int StepNo = 250, tempstepno;
char c, *str2;
BOOL r_abort;     // Rendering abort flag
BOOL repeat;      // Repeat flag
BOOL status;      // Status flag
BOOL rendered=FALSE;
BOOL shaded=FALSE;
static BOOL par_flag = FALSE;     // Parallel projection flag
static int c_type;    // Color type
static BOOL r_flag;   // Color resolution flag

/////////////////////////////////////////////////////////////////////////////
void callback_form12_OK(FL_OBJECT *obj, long foo) {
          fl_hide_form(form12->About);
}
/////////////////////////////////////////////////////////////////////////////
// Set camera parameters
void initialize_setcamera()
{
  double vpd, fpd, bpd;     // Camera distances
  double scale;             // Window scaling factor
  double tilt;              // Camera tilt angle
  int w, h;                 // Bitmap window dimensions
  BOOL dummy;               // Dummy parameter

  char str[25];
  double num;
  int sig = 5; /* significant digits */

	// Get camera distances     /* FLOAT YAP */
	gcvt(Camera.GetViewDist(), sig, str);
	fl_set_input(form7->view, str);
	gcvt(Camera.GetFrontDist(), sig, str);
	fl_set_input(form7->front_plane, str);
	gcvt(Camera.GetBackDist(), sig, str);
	fl_set_input(form7->back_plane, str);

	// Get window scaling factor   /* FLOAT YAP */
	gcvt(Camera.GetWindowScale(), sig, str);
	fl_set_input(form7->scaling_factor, str);

	// Get bitmap window dimensions  /* INT YAP false*/
	gcvt(Camera.GetWidth(), sig, str);
	fl_set_input(form7->horizontal_pixels, str);
	gcvt(Camera.GetHeight(), sig, str);
	fl_set_input(form7->vertical_pixels, str);

	// Get camera tilt angle  /* FLOAT YAP */
	gcvt(RadToDeg(Camera.GetTiltAngle()), sig, str);
	fl_set_input(form7->degrees, str);

	// Get projection type
	par_flag = Camera.GetParProjFlag();

	if (par_flag == TRUE){
	   callback_parallel(form7->parallel,0);
	}
	else{
	   callback_perspective(form7->perspective,0);
	}
}
/////////////////////////////////////////////////////////////////////////////
// Set camera view parameters
void initialize_setview ()
{
  Point3 eye_posn;      // Camera eye position
  Point3 focus_posn;    // Camera focus position
  Vector3 temp;         // Temporary vector

  char str[25];
  double num;
  int sig = 5; /* significant digits */

      // Get camera eye position
      eye_posn = Camera.GetEyePosn();       
      gcvt(eye_posn.GetX(), sig, str);
      fl_set_input(form8->eye_x_axis, str);
		gcvt(eye_posn.GetY(), sig, str);
      fl_set_input(form8->eye_y_axis, str);
      gcvt(eye_posn.GetZ(), sig, str);
      fl_set_input(form8->eye_z_axis, str);

		// Get camera focus position
      focus_posn = Camera.GetFocusPosn();
      gcvt(focus_posn.GetX(), sig, str);
      fl_set_input(form8->focus_x_axis, str);
      gcvt(focus_posn.GetY(), sig, str);
      fl_set_input(form8->focus_y_axis, str);
      gcvt(focus_posn.GetZ(), sig, str);
      fl_set_input(form8->focus_z_axis, str);
}

/////////////////////////////////////////////////////////////////////////////
// Set radiosity rendering convergence parameters
void initialize_setconvergence()
{
  int mp;       // Maximum number of steps
  double sc;    // Stopping criterion
  BOOL dummy;   // Dummy parameter

  char str[25];
  double num;
  int sig = 5; /* significant digits */   

      sprintf(str, "%d", Radiosity.GetMaxStep());
      fl_set_input(form5->maximum_steps, str);

      gcvt(Radiosity.GetStopCriterion(), sig, str);
		fl_set_input(form5->stopping_criterion, str);
      if (Radiosity.AmbientFlag())
	 fl_set_button(form5->ambient_exitance, 1);
      else
	 fl_set_button(form5->ambient_exitance, 0);

      if (Radiosity.OverShootFlag())
         fl_set_button(form5->positive_overshoot, 1);
}
/////////////////////////////////////////////////////////////////////////////
// Set display parameters
void initialize_setdisplay ()
{
  int i;                // Loop index
  int noise;            // Noise parameter
  double contrast;      // Contrast value
  double gamma;         // Gamma parameter
  double intensity;     // Intensity value
  BOOL dummy;           // Dummy parameter

  char str[25],c;
  double num;
  int sig = 5; /* significant digits */

      gcvt(Camera.GetIntensity(), sig, str);
      fl_set_input(form3->intensity_factor, str);
		gcvt(Camera.GetContrast(), sig, str);
      fl_set_input(form3->contrast_factor, str);
      gcvt(Camera.GetGamma(), sig, str);
      fl_set_input(form3->gamma_factor, str);

      sprintf(str, "%d", Camera.GetNoiseLevel());
      fl_set_input(form3->color_reduction_noise_level, str);

      if (Camera.IntensityFlag())
         fl_set_button(form3->intensity_enabled, 1);
      else
         fl_set_button(form3->intensity_enabled, 0);
 
      if (Camera.ContrastFlag())
         fl_set_button(form3->contrast_enabled, 1);
      else
         fl_set_button(form3->contrast_enabled, 0);

      if (Camera.GammaFlag())
         fl_set_button(form3->gamma_enabled, 1);
      else
         fl_set_button(form3->gamma_enabled, 0);

      if (Camera.JitterFlag())
         fl_set_button(form3->color_reduction_enabled, 1);
      else
         fl_set_button(form3->color_reduction_enabled, 0);
/* 
		// Initialize filter selection combo box
      for (i = 0; i < Camera.GetNumFilters(); i++)
        SendDlgItemMessage(hdlg, IDC_FILTERSEL,
            CB_INSERTSTRING, (WPARAM) i, (LPARAM)
      
	Camera.GetFilterSize(i));

      // Select current antialiasing filter
      (void) SendDlgItemMessage(hdlg, IDC_FILTERSEL,
          CB_SELECTSTRING, (WPARAM) -1, (LPARAM)
          Camera.GetCurrFilterSize());

      CenterDialog(hdlg);
      return TRUE;
	 case WM_COMMAND:
*/     
}

/////////////////////////////////////////////////////////////////////////////
void initialize_setcolor()
{/* 
        c_type = Camera.GetColorType();
//      r_flag = Bitmap.GetRGBFlag();

      if (c_type == SC_RGB){
	 fl_set_input(form4->rgb_color,1);
	 fl_set_input(form4->grayscale,0);
	 fl_set_input(form4->pseudocolor,0);
      }  
      else if (c_type == SC_MONO){
	 fl_set_input(form4->rgb_color,0);
	 fl_set_input(form4->grayscale,1);
	 fl_set_input(form4->pseudocolor,0);
      }
      else{
	 fl_set_input(form4->rgb_color,0);
	 fl_set_input(form4->grayscale,0);
	 fl_set_input(form4->pseudocolor,1);
      }
      CheckRadioButton(hdlg, IDC_RGB, IDC_PSEUDO, c_type +
			 IDC_RGB);

  return FALSE; */
}
/********************** CALLBACKS OF OK BUTTONS ************************/
/////////////////////////////////////////////////////////////////////////////
void callback_form2_OK(FL_OBJECT *obj, long foo) {
	  EntityDir = (char *)fl_get_input(form2->entities_file_path);
	  fl_hide_form(form2->Entities);
}
/////////////////////////////////////////////////////////////////////////////
void callback_form3_OK(FL_OBJECT *obj, long foo) {
	callback_form3_UPDATE(form3->form3_UPDATE, 0);
	  fl_hide_form(form3->Set_Display);
}
/////////////////////////////////////////////////////////////////////////////
void callback_form4_OK(FL_OBJECT *obj, long foo) {
	callback_form4_UPDATE(form4->form4_UPDATE, 0);
	  fl_hide_form(form4->Set_Color);
}
/////////////////////////////////////////////////////////////////////////////
void callback_form6_OK(FL_OBJECT *obj, long foo) {
	  fl_hide_form(form6->Statistics);
}
/////////////////////////////////////////////////////////////////////////////
void callback_form7_OK(FL_OBJECT *obj, long foo) {
	callback_form7_UPDATE(form7->form7_UPDATE, 0);
	  fl_hide_form(form7->Set_Camera);
}
/////////////////////////////////////////////////////////////////////////////
void callback_form8_OK(FL_OBJECT *obj, long foo) {
	callback_form8_UPDATE(form8->form8_UPDATE, 0);
	fl_hide_form(form8->Set_View);
}
/////////////////////////////////////////////////////////////////////////////
void callback_form11_OK(FL_OBJECT *obj, long foo) {
	  fl_hide_form(form11->errormessage);
}
/////////////////////////////////////////////////////////////////////////////
void callback_perspective(FL_OBJECT *obj, long foo) {
     fl_set_button(form7->perspective,1);
     fl_set_button(form7->parallel,0);
}
/////////////////////////////////////////////////////////////////////////////
void callback_parallel(FL_OBJECT *obj, long foo) {
     fl_set_button(form7->parallel,1);
     fl_set_button(form7->perspective,0);
}
/////////////////////////////////////////////////////////////////////////////
void callback_rgb_color(FL_OBJECT *obj, long foo) {
     fl_set_button(form4->rgb_color,1);
     fl_set_button(form4->grayscale,0);
     fl_set_button(form4->pseudocolor,0);
}
/////////////////////////////////////////////////////////////////////////////
void callback_grayscale(FL_OBJECT *obj, long foo) {
     fl_set_button(form4->rgb_color,0);
     fl_set_button(form4->grayscale,1);
     fl_set_button(form4->pseudocolor,0);
}
/////////////////////////////////////////////////////////////////////////////
void callback_pseudocolor(FL_OBJECT *obj, long foo) {
     fl_set_button(form4->rgb_color,0);
     fl_set_button(form4->grayscale,0);
     fl_set_button(form4->pseudocolor,1);
}
/////////////////////////////////////////////////////////////////////////////
void ReadAndDisplayImage(char *filename) {
  int y;

  in = tiff_read(filename, NULL);
  if( !in ) {
	 printf("Unable to open input file '%s'\n", filename);
	 exit(1);
  }

  if ((in->nx != WIDTH) || (in->ny != HEIGHT)) {
	 printf("Image must be %dx%d\n", WIDTH, HEIGHT);
	 exit(1);
  }

  printf("Read a %dx%d TIFF image (%d bytes per pixel)\n",
	 in->nx, in->ny, in->bpp);

  for (y = 0; y < HEIGHT; y++) {
    glRasterPos2i(0, y);
	 glDrawPixels( WIDTH, 1, GL_RGB, GL_UNSIGNED_BYTE,
		  &(PIC_PIXEL(in, 0, y, 0)));}

}

/////////////////////////////////////////////////////////////////////////////
void WriteImage(char *filename) {
  tiff_write(filename, in);
}

/////////////////////////////////////////////////////////////////////////////
void shootpart(){
int i;
     
//        glClear(GL_COLOR_BUFFER_BIT);
  //      glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,tbuffer);
	r_abort = FALSE;i=0;
	do
	{
	 if ((status = Camera.Shoot(penv, WIDTH,HEIGHT, &repeat)) == FALSE)
	    break;
	 printf("SHOOTA GIRDI\n");
	}while (repeat == TRUE);
	if ((status==TRUE)&&(r_abort==FALSE))
	for (i = 0; i < HEIGHT; i++) {
	    glRasterPos2i(0, HEIGHT-1-i);
	    glDrawPixels( WIDTH, 1, GL_RGB, GL_UNSIGNED_BYTE, &tbuffer[(i)*WIDTH*3]);
	}
	glFlush();
}
/////////////////////////////////////////////////////////////////////////////
void callback_refresh(FL_OBJECT *obj, long foo){
int i;
     glClear(GL_COLOR_BUFFER_BIT);
     for (i = 0; i < HEIGHT; i++) {
	glRasterPos2i(0, HEIGHT-1-i);
	glDrawPixels( WIDTH, 1, GL_RGB, GL_UNSIGNED_BYTE, &tbuffer[(i)*WIDTH*3]);}
}
/////////////////////////////////////////////////////////////////////////////    
void openFilepart(){
char *myfile;
  int i,row,col,j,k=0;  char a,c;              // Loop index
  int num_vert;         // Number of vertices
  Element3 *pelem;      // Element pointer
  Instance *pinst;      // Instance pointer
  OutPolygon out;       // Output polygon
  POINT vertex[8];      // Polygon vertex array
  Point3 posn;          // Point co-ordinates
  Patch3 *ppatch;       // Patch pointer
  Surface3 *psurf;      // Surface pointer
  FILE *in;  

     glClearColor(0.9,0.9,1,0);
     glClear(GL_COLOR_BUFFER_BIT);
     fl_set_fselector_placement(FL_PLACE_POSITION);
     myfile=(char *)fl_show_fselector("Please enter the filename to open",
     "/csfaculty/gudukbay/helios_sgi/helios/proje","*.wld","room.wld");
     glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,tbuffer);
	  if ((myfile!=NULL)&&(1)){ //
	if(in=fopen(myfile,"r")){

	   if ((Parser.ParseFile(myfile,
                "/csfaculty/gudukbay/helios_sgi/helios/proje",
                &Environment)) == TRUE){

		// Initialize camera view system
		penv=&Environment;
		Camera.InitViewSystem(penv);

		// Display wireframe

		  pinst = penv->GetInstPtr();
		  while (pinst != NULL)
  		  {
  			  psurf = pinst->GetSurfPtr();
   			 while (psurf != NULL)
    			 {
      			 ppatch = psurf->GetPatchPtr();
					 while (ppatch != NULL)
					{
      			   if (Camera.BackFaceCull(ppatch) == FALSE)
    			    {
         			 // Walk the element list
         			 pelem = ppatch->GetElementPtr();
						 while (pelem != NULL)
      			         {
					  // Clip the 3-D element (polygon)
          			  num_vert = clipper.Clip(pelem, out,
						Camera.GetProjMatrix());

      			      // Initialize the 2-D polygon vertices array
					for (i = 0; i < num_vert; i++)
					{
          		      		  posn  = out.GetVertexPosn(i);
	
	        	          // Convertnormalizeddeviceco-ordinates to
	             		  // screen space co-ordinates
        	       		   vertex[i].x = (int)(posn.GetX()* WIDTH);
				vertex[i].y =240 - (int)(posn.GetY()*HEIGHT);
					  }
				//***************
				for (i=0;i<num_vert-1;i++)
				{
			 	glBegin(GL_LINES);
				glVertex2i(vertex[i].x,vertex[i].y);
				glVertex2i(vertex[i+1].x,vertex[i+1].y);
				glEnd();
				} 
				//****************
					   
	                	   pelem = pelem->GetNext();
					 }
					}//end of if


				ppatch =ppatch->GetNext();

      			}
					if (j==2) break;

      			psurf = psurf->GetNext();
					if (j==2) break;

    		     }
			if((pinst->GetNext()!=NULL)&&((j==0)||(j==1)))	
   		     	pinst = pinst->GetNext();
			  else if(j==2) { pinst=NULL;
					 break;
			 }
		     else j++;

		 }
		// Reset radiosity calculations done flag

		   RadCalcDone = FALSE;
              fl_set_menu(form->render_menu,"Wireframe|Shaded|Rendering|%dRedisplay");

	     }

	}
		fclose(in);
	 }
     glFlush();
     glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,buffer);
     glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,tbuffer);
     glClearColor(0,0,0,0);
     glClear(GL_COLOR_BUFFER_BIT);
}

/////////////////////////////////////////////////////////////////////////////
void saveFilepart(){
int y;
char *myfile;

	fl_set_fselector_placement(FL_PLACE_POSITION);
	myfile=(char *)fl_show_fselector("Save As",
               "/csfaculty/gudukbay/helios_sgi/helios/proje/",
               "*.tif","out.tif");
	for(y=0; y<HEIGHT;  y++){
		glReadPixels(0,y,WIDTH,1, GL_RGB, GL_UNSIGNED_BYTE,
			&(PIC_PIXEL(in, 0, HEIGHT-y, 0)));}
	if (myfile!=NULL) WriteImage(myfile);
}

/////////////////////////////////////////////////////////////////////////////
void entitiespart() {
  form2 = create_form_Entities();
  fl_set_form_position(form2->Entities, 310, 340);
  fl_show_form(form2->Entities, FL_PLACE_POSITION, FL_TRANSIENT,
"Entities Directory");
}

/////////////////////////////////////////////////////////////////////////////
void setdisplaypart(){
  form3 = create_form_Set_Display();
  fl_set_form_position(form3->Set_Display, 310, 340);
  fl_show_form(form3->Set_Display, FL_PLACE_POSITION, FL_TRANSIENT,
"Display Parameters");
  initialize_setdisplay();   
}
/////////////////////////////////////////////////////////////////////////////
void setcolorpart(){
  form4 = create_form_Set_Color();
  fl_set_form_position(form4->Set_Color, 310, 340);
  fl_show_form(form4->Set_Color, FL_PLACE_POSITION, FL_TRANSIENT,
"Color Parameters");
  initialize_setcolor();  
}
/////////////////////////////////////////////////////////////////////////////
void setconvergencepart(){
  form5 = create_form_Set_Convergence();
  fl_set_form_position(form5->Set_Convergence, 310, 340);
  fl_show_form(form5->Set_Convergence, FL_PLACE_POSITION,
FL_TRANSIENT,"Convergence Parameters");
  initialize_setconvergence();  
}
/////////////////////////////////////////////////////////////////////////////
void callback_optionsMenu(FL_OBJECT *obj, long foo) {
int itemNo;
  itemNo = fl_get_menu(form->options_menu);
  if (itemNo == 1) setdisplaypart() ;
  else if (itemNo == 2)  setcolorpart();
  else if (itemNo == 3)  setconvergencepart();
}
/////////////////////////////////////////////////////////////////////////////
void Entities_Cancel(FL_OBJECT *obj, long foo){
  fl_hide_form(form2->Entities);
 // refreshOUT();
}
/////////////////////////////////////////////////////////////////////////////
void Statistics_OK(FL_OBJECT *obj, long foo){
  fl_hide_form(form6->Statistics);
}
/////////////////////////////////////////////////////////////////////////////
void Set_Convergence_Cancel(FL_OBJECT *obj, long foo){
  fl_hide_form(form5->Set_Convergence);
}
/////////////////////////////////////////////////////////////////////////////
void Set_Color_Cancel(FL_OBJECT *obj, long foo){
  fl_hide_form(form4->Set_Color);
}
/////////////////////////////////////////////////////////////////////////////
void Set_Display_Cancel(FL_OBJECT *obj, long foo){
  fl_hide_form(form3->Set_Display);
}
/////////////////////////////////////////////////////////////////////////////
void Set_View_Cancel(FL_OBJECT *obj, long foo){
fl_hide_form(form8->Set_View);
}
/////////////////////////////////////////////////////////////////////////////
void Set_Camera_Cancel(FL_OBJECT *obj, long foo){
  fl_hide_form(form7->Set_Camera);
}
/////////////////////////////////////////////////////////////////////////////
void setcamerapart(){
  form7 = create_form_Set_Camera();
  fl_set_form_position(form7->Set_Camera, 310, 340);
  fl_show_form(form7->Set_Camera, FL_PLACE_POSITION, FL_TRANSIENT,
"Camera Parameters");
  initialize_setcamera();    
}
/////////////////////////////////////////////////////////////////////////////
void setviewpart(){
  form8 = create_form_Set_View();
  fl_set_form_position(form8->Set_View, 310, 340);
  fl_show_form(form8->Set_View, FL_PLACE_POSITION, FL_TRANSIENT,
"View Parameters");
  initialize_setview(); 
}
/////////////////////////////////////////////////////////////////////////////
void callback_cameraMenu(FL_OBJECT *obj, long foo) {
int itemNo;
  itemNo = fl_get_menu(form->camera_menu);
  if (itemNo == 1) setcamerapart();
  else if (itemNo == 2) setviewpart();
}

/////////////////////////////////////////////////////////////////////////////
void Renderpart(){
int i=0;
int row,col;
char str[25];
int sig = 5; /* significant digits */

       shaded=FALSE;
       rendered=TRUE;
       tempstepno = StepNo;
       glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,tbuffer);
			// Reset radiosity calculations done flag
	 RadCalcDone = FALSE;
	// Confirm radiosity equation solver status
		if (Radiosity.GetStatus() == FALSE){
			//  OutOfMemory();
			printf("out of memory!");// break;
		}
		// Initialize equation solver
		if (Radiosity.Open(penv /*&Environment*/) == FALSE){
			printf("out of memory");// OutOfMemory();
				//EnableMenu(hmenu);    // Enable menu items
		}
			//** Maximum step ve stop criterion aliniyor?
		InCalc = TRUE;
			// ***************************
		while (Radiosity.Calculate() == FALSE){
		   //  if (tempstepno > 0) tempstepno--;
		   //  else { AbortFlag=FALSE; break;}

                     if (tempstepno==0) {
			AbortFlag=FALSE;
			break;
		     }
		     InCalc = FALSE; ////user aborshoul be initialized
		     Radiosity.SetUpdate();  // get environment updates     
		 //    glClear(GL_COLOR_BUFFER_BIT);
  		     shootpart();
		     if (Camera.AntiAliasFlag() == TRUE){
		        InCalc = TRUE; 
		        printf("ANTIALIASIN ICINE GIRDI\n");
		      }
                     tempstepno--;
                  //   glFlush();
	       }/* while calculate */
     Radiosity.Close();
  //   glClear(GL_COLOR_BUFFER_BIT);
     shootpart();
    // glFlush();
     printf("Calculate ends...\n");
}
/////////////////////////////////////////////////////////////////////////////
void callback_form10_OK(FL_OBJECT *obj, long foo) {
char str[25];
int sig = 5; /* significant digits */
  
  StepNo = atoi(fl_get_input(form10->stepno));
  if (StepNo < 1 || StepNo > 250){
  	form11 = create_form_errormessage();
	fl_set_form_position(form11->errormessage, 310, 340);
  	fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
	fl_set_object_label(form11->errortext,
           "Step no must be between 1 and 250.");
        return;
  }
  else{
  fl_hide_form(form10->getstepno);
  Renderpart();
  form9 = create_form_Calculate();
  fl_set_form_position(form9->Calculate, 310, 340);
  fl_show_form(form9->Calculate, FL_PLACE_POSITION, FL_TRANSIENT,
                "Radiosity Calculations");

  sprintf(str, "%d", Radiosity.GetStepCount());
  fl_set_object_label(form9->number_of_steps, str);

  gcvt(Radiosity.GetConvergence(), sig, str);
  fl_set_object_label(form9->convergence, str);
  fl_set_object_label(form9->elapsed_time,
                  Radiosity.GetElapsedTime());
  }
}
/////////////////////////////////////////////////////////////////////////////
void callback_form9_OK(FL_OBJECT *obj, long foo) {
	   fl_hide_form(form9->Calculate);
}
/////////////////////////////////////////////////////////////////////////////
void shadepart(){
int i;
  shaded=TRUE;
  rendered=FALSE;
  glClear(GL_COLOR_BUFFER_BIT);
  glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,tbuffer);
  Radiosity.Shade(penv);
  RadCalcDone = FALSE;
		r_abort = FALSE;i=0;
		do
		{
			if ((status = Camera.Shoot(penv, WIDTH,HEIGHT,
				&repeat)) == FALSE)
			break;

		if (Camera.AntiAliasFlag() == TRUE)
		{
			// Update dialog box
//      SetDlgItemInt(hAbortDlg, IDC_NUMPASS,
	//        Camera.GetNumPass(), FALSE);

			// Check for user-requested abort
	/*    if (CheckAbort() == TRUE)
			{
				r_abort = TRUE;
				AbortFlag = FALSE;
				Camera.Abort();
				break;
			}*/
         scanf("%c",&c);
	 if (c=='a'){ r_abort=TRUE;AbortFlag=FALSE;/*Camera.Abort();*/
		break; }

	 }
	}while (repeat == TRUE);
		if ((status==TRUE)&&(r_abort==FALSE))
		for (i = 0; i < HEIGHT; i++) {
			 glRasterPos2i(0, HEIGHT-1-i);
			 glDrawPixels( WIDTH, 1, GL_RGB, GL_UNSIGNED_BYTE,
                                       &tbuffer[(i)*WIDTH*3]);}
		glFlush();

}
/////////////////////////////////////////////////////////////////////////////
void callback_form7_UPDATE(FL_OBJECT *obj, long foo) {
double vpd, fpd, bpd;     // Camera distances
double scale;             // Window scaling factor
double tilt;              // Camera tilt angle
int w, h;                 // Bitmap window dimensions

	  vpd = atof(fl_get_input(form7->view));
          fpd = atof(fl_get_input(form7->front_plane));
          bpd = atof(fl_get_input(form7->back_plane));

	  if (fl_get_button(form7->perspective)==1) par_flag=FALSE;
	  else if (fl_get_button(form7->parallel)==1) par_flag=TRUE;

          if (vpd <= 0.0)
          {
  	    form11 = create_form_errormessage();
	    fl_set_form_position(form11->errormessage, 310, 340);
  	    fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
	    fl_set_object_label(form11->errortext,
               "View distance must be greater than zero.");
            return;
          }                                               

          if (fpd >= bpd)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Front distance must be smaller than back distance.");
            return;
          }      

	  if (fpd < -(vpd - MIN_VALUE))
	  {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Eye distance error occured.");
            return;
          }

          // Validate window scaling factor  FLOAT 
          scale = atof(fl_get_input(form7->scaling_factor));

          if (scale <= 0.0) 
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
	    fl_set_object_label(form11->errortext,
               "Scale must be greater than zero.");
            return;
	  }

          // Validate bitmap window dimensions
          w = atoi(fl_get_input(form7->horizontal_pixels));
	  h = atoi(fl_get_input(form7->vertical_pixels));

	  if (w < MinPixel || w > MaxPixel || h < MinPixel
              || h > MaxPixel)
          {

            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, 
               FL_PLACE_POSITION, FL_TRANSIENT, " ");
	    fl_set_object_label(form11->errortext,
"Number of horizontal and vertical pixels must be between 32 and 1024.");
            return;
          }    

          // Validate tilt angle        double =? float 
          tilt = atof(fl_get_input(form7->degrees)); 

	  if (tilt < 0.0 || tilt > 360.0)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Tilt angle must be between 0 and 360.");
            return;
          }
                       
 	 // Set distances
	  Camera.SetViewDist(vpd);
	  Camera.SetFrontDist(fpd);
          Camera.SetBackDist(bpd);

          // Set window scaling factor
          Camera.SetWindowScale(scale);

          // Set bitmap window dimensions
          Camera.SetWidth(w);
                                 
          Camera.SetHeight(h);

          // Set camera tilt angle
	  Camera.SetTiltAngle(DegToRad(tilt));

          // Set projection type
          Camera.SetParProjFlag(par_flag);

          // Update view system parameters
	  Camera.UpdateViewSystem();

          if (rendered==TRUE){
	     Radiosity.GetStatus();
/*       	     Radiosity.Open(penv);
//	     Radiosity.Calculate();             
*/	     Radiosity.Close();
	     shootpart();
	  }
          else if (shaded==TRUE) shadepart();
}
/////////////////////////////////////////////////////////////////////////////
void callback_form8_UPDATE(FL_OBJECT *obj, long foo){
  Point3 eye_posn;      // Camera eye position
  Point3 focus_posn;    // Camera focus position
  Vector3 temp;         // Temporary vector

	// Get eye position
	
        eye_posn.SetX(atof(fl_get_input(form8->eye_x_axis)));
        eye_posn.SetY(atof(fl_get_input(form8->eye_y_axis)));
        eye_posn.SetZ(atof(fl_get_input(form8->eye_z_axis)));

        // Get focus position
        focus_posn.SetX(atof(fl_get_input(form8->focus_x_axis)));
        focus_posn.SetY(atof(fl_get_input(form8->focus_y_axis)));
        focus_posn.SetZ(atof(fl_get_input(form8->focus_z_axis)));

          // Validate eye and focus positions
	temp = Vector3(eye_posn, focus_posn);
        if (fabs(temp.Length()) < MIN_VALUE)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,"Focus error occured.");
            return;
          }

          // Set camera view parameters
          Camera.SetEyeFocusPosn(eye_posn, focus_posn);

          // Update view system parameters
          Camera.UpdateViewSystem();

          // Instruct parent window to update client area
                 
          if (rendered==TRUE){
	     Radiosity.GetStatus();
/*       	     Radiosity.Open(penv);
//	     Radiosity.Calculate();             
*/             Radiosity.Close();
	     shootpart();
	  }
          else if (shaded==TRUE) shadepart();
}
////////////////////////////////////////////////////////////////
void callback_form5_OK(FL_OBJECT *obj, long foo){
int mp;       // Maximum number of steps
double sc;    // Stopping criterion
BOOL dummy;   // Dummy parameter

          // Validate parameters
          mp = atoi(fl_get_input(form5->maximum_steps));
          sc = atof(fl_get_input(form5->stopping_criterion));
          if (mp < 1 || mp > MaxStep)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Maximum step no must be between 1 and 2000.");
            return;
          }
          if (sc <= 0.0 || sc > 1.0)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Stopping criterion must be in (0,1).");
            return;
	  }
          // Set convergence parameters
          Radiosity.SetMaxStep(mp);
	  Radiosity.SetStopCriterion(sc);
	  if (fl_get_button(form5->ambient_exitance)==1)
            Radiosity.EnableAmbient();
	  else Radiosity.DisableAmbient();
	  if (fl_get_button(form5->positive_overshoot)==1)
            Radiosity.EnableOverShoot();
	  else
            Radiosity.DisableOverShoot();
	  fl_hide_form(form5->Set_Convergence);
          if (rendered==TRUE){
	     Radiosity.GetStatus();
/*       	     Radiosity.Open(penv);
//	     Radiosity.Calculate();             
*/             Radiosity.Close();
	     shootpart();
	  }
          else if (shaded==TRUE) shadepart();
}
////////////////////////////////////////////////////////////////
void callback_form3_UPDATE(FL_OBJECT *obj, long foo){
  int i;                // Loop index
  int noise;            // Noise parameter
  double contrast;      // Contrast value
  double gamma;         // Gamma parameter
  double intensity;     // Intensity value
  BOOL dummy;           // Dummy parameter

          intensity = atof(fl_get_input(form3->intensity_factor));

          // Set intensity parameter
          Camera.SetIntensity(intensity);

          contrast = atof(fl_get_input(form3->contrast_factor));

          if (contrast <= 0.0)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Contrast must be greater than 0");
            return;
          }

          // Set contrast parameter
	  Camera.SetContrast(contrast);

          gamma = atof(fl_get_input(form3->gamma_factor));

          if (gamma <= 0.0)
	  {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Gamma must be greater than 0");
            return;
          }

          // Set gamma correction parameter
          Camera.SetGamma(gamma);

          noise = atoi(fl_get_input(form3->color_reduction_noise_level));
 
          if (noise < 0 || noise > 8)
          {
            form11 = create_form_errormessage();
            fl_set_form_position(form11->errormessage, 310, 340);
            fl_show_form(form11->errormessage, FL_PLACE_POSITION,
FL_TRANSIENT, " ");
            fl_set_object_label(form11->errortext,
               "Noise level must be between 0 and 8.");
            return;
          }

	  // Set noise level parameter
          Camera.SetNoiseLevel(noise);
  
/*          // Set antialiasing filter type
          Camera.SetFilterType((int)
              SendDlgItemMessage(hdlg, IDC_FILTERSEL,
              CB_GETCURSEL, (WPARAM) 0, (LPARAM) 0L));
*/
	  if (fl_get_button(form3->intensity_enabled)==1)
            Camera.EnableIntensity();
          else
            Camera.DisableIntensity();

          if (fl_get_button(form3->contrast_enabled) == 1)
            Camera.EnableContrast();
          else
            Camera.DisableContrast();

          if (fl_get_button(form3->gamma_enabled) == 1)
            Camera.EnableGamma();
          else
            Camera.DisableGamma();
   
          if  (fl_get_button(form3->color_reduction_enabled) == 1)
            Camera.EnableJitter();
          else
	    Camera.DisableJitter();

          if (rendered==TRUE){
	     Radiosity.GetStatus();
/*       	     Radiosity.Open(penv);
//	     Radiosity.Calculate();             
*/             Radiosity.Close();
	     shootpart();
	  }
          else if (shaded==TRUE) shadepart();
}

/////////////////////////////////////////////////////////////////////////////
void callback_form4_UPDATE(FL_OBJECT *obj, long foo){

	if (fl_get_button(form4->rgb_color)==1) c_type = SC_RGB;
        else if (fl_get_button(form4->grayscale)==1) c_type = SC_MONO;
        else if (fl_get_button(form4->pseudocolor)==1) c_type = SC_PSEUDO;

          // Set display color type
          Camera.SetColorType(c_type);

          // Set display color resolution
//          Bitmap.SetRGBFlag(r_flag);

          if (rendered==TRUE){
	     Radiosity.GetStatus();
/*       	     Radiosity.Open(penv);
//	     Radiosity.Calculate();             
*/             Radiosity.Close();
	     shootpart();
	  }
          else if (shaded==TRUE) shadepart();
}
/////////////////////////////////////////////////////////////////////////////
void statisticspart() {
  form6 = create_form_Statistics();
  fl_set_form_position(form6->Statistics, 310, 340);
  fl_show_form(form6->Statistics, FL_PLACE_POSITION, FL_TRANSIENT,
"Environment Statistics");
      // Initialize environment statistics

  char str[25];

  sprintf(str, "%d", Environment.GetNumInst());

  fl_set_object_label(form6->instances, str);

  sprintf(str, "%d", Environment.GetNumSurf());

  fl_set_object_label(form6->surfaces, str);

  sprintf(str, "%d", Environment.GetNumPatch());

  fl_set_object_label(form6->patches, str);

  sprintf(str, "%d", Environment.GetNumElem());

  fl_set_object_label(form6->elements, str);

  sprintf(str, "%d", Environment.GetNumVert());

  fl_set_object_label(form6->vertices, str);

}
////////////////////////////////////////////////////////////////
void calculatepart(){
  
  form10 = create_form_getstepno();
  fl_set_form_position(form10->getstepno, 310, 340);
  fl_show_form(form10->getstepno, FL_PLACE_POSITION, FL_TRANSIENT, " ");
  fl_set_input(form10->stepno, "250");
}
 
/////////////////////////////////////////////////////////////////////////////
void wireframepart(){
int i;
	for (i = 0; i < HEIGHT; i++) {
            glRasterPos2i(0, HEIGHT-1-i);
	    glDrawPixels( WIDTH, 1, GL_RGB, GL_UNSIGNED_BYTE, &buffer[(i)*WIDTH*3]);
	}
        glFlush();
        glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,tbuffer);
}
/////////////////////////////////////////////////////////////////////////////
void callback_renderMenu(FL_OBJECT *obj, long foo) {
int itemNo;char c;
	itemNo = fl_get_menu(form->render_menu);
	if (itemNo == 1) wireframepart();
	else if (itemNo == 2) shadepart();
	else if (itemNo == 3) calculatepart();
}
/////////////////////////////////////////////////////////////////////////////
void callback_fileMenu(FL_OBJECT *obj, long foo) {
int itemNo;char c;
  itemNo = fl_get_menu(form->file_menu);
  if (itemNo == 1) openFilepart();
  else if (itemNo == 2) saveFilepart();
  else if (itemNo == 3) entitiespart();
  else if (itemNo == 4) statisticspart();
}

/////////////////////////////////////////////////////////////////////////////
void callback_helpMenu(FL_OBJECT *obj, long foo) {
	printf("girdi");
}

/////////////////////////////////////////////////////////////////////////////
void callback_interactive(){
  printf("interactive");
}

/////////////////////////////////////////////////////////////////////////////
void callback_dolly(){
  printf("dolly");
}

/////////////////////////////////////////////////////////////////////////////
void callback_orbit(){
  printf("orbit");
}

/////////////////////////////////////////////////////////////////////////////
void callback_pan(){
  printf("pan");
}

/////////////////////////////////////////////////////////////////////////////
void callback_rotate(){
  printf("rotate");
}

/////////////////////////////////////////////////////////////////////////////
void callback_tilt(){
  printf("tilt");
}

/////////////////////////////////////////////////////////////////////////////
void callback_zoom(){
  printf("zoom");
}

/////////////////////////////////////////////////////////////////////////////
void InitializeMenus(){
  fl_set_menu(form->file_menu,"Open...|SaveAs...%l|Entities...%l|Statistics");
  fl_set_menu(form->camera_menu,"Set Camera...|Set View...");
  fl_set_menu(form->render_menu,"%dWireframe|%dShaded|%dRendering");
  fl_set_menu(form->options_menu,"Set Display...|Set Color...|Set Convergence...");
  fl_set_menu(form->about_menu, "");
}
/////////////////////////////////////////////////////////////////////////////
void callback_aboutMenu(FL_OBJECT *obj, long foo) {
  form12 = create_form_About();
  fl_set_form_position(form12->About, 310, 340);
  fl_show_form(form12->About, FL_PLACE_POSITION, FL_TRANSIENT, " ");
}
/////////////////////////////////////////////////////////////////////////////
void handle_event(XEvent *event) {
	/* see /usr/include/X11/X.h for definitions of X Windows event types */

  switch (event->type) {
  case ButtonPress:    /* button press */
    mouse.button = event->xbutton.button;
    mouse.x = event->xbutton.x/ZOOM;
	 mouse.y = event->xbutton.y/ZOOM;
    printf("ButtonPress.  button %d at (%d, %d)\n",
	   mouse.button, mouse.x, mouse.y);
    break;

  case ButtonRelease:  /* button release */
    printf("ButtonRelease. button %d\n", event->xbutton.button);
    mouse.button = 0;
    break;

  case MotionNotify:   /* mouse pointer moved */
    printf("MotionNotify at (%d, %d) state=%d=(%s %s %s)\n",
	event->xmotion.x/ZOOM, event->xmotion.y/ZOOM,
	event->xmotion.state,
	event->xmotion.state&Button1Mask ? "button1" : ".",
	event->xmotion.state&Button2Mask ? "button2" : ".",
	event->xmotion.state&Button3Mask ? "button3" : ".");
    break;

  case EnterNotify:       /* mouse pointer entered window */
    printf("EnterNotify.\n");
    break;

  case LeaveNotify:       /* mouse pointer left window */
    printf("LeaveNotify.\n");
    break;

  case KeyPress:          /* key pressed */
  case KeyRelease:        /* key released */
    {
	char buf[10];
	int n;
	KeySym keysym;

	printf(event->type==KeyPress ? "KeyPress." : "KeyRelease.");
	n = XLookupString(&event->xkey, buf, sizeof buf - 1, &keysym, 0);
	buf[n] = 0;
	/* print key as ascii and in hexadecimal */
	/* see /usr/include/X11/keysymdef.h for key codes */
	printf(" key=(%c)=0x%x\n", keysym, keysym);
    }
    break;

  case Expose:            /* window just uncovered */
    printf("Expose.\n");
    break;

  case VisibilityNotify:  /* window occluded or exposed */
    printf("VisibilityNotify.\n");
    break;

  default:
		printf("unknown event type.\n");
		break;
  }
}

/////////////////////////////////////////////////////////////////////////////
void error_check(int loc) {
  /* this routine checks to see if OpenGL errors have occurred recently */
  GLenum e;

	while ((e = glGetError()) != GL_NO_ERROR)
		fprintf(stderr, "Error: %s before location %d\n", gluErrorString(e), loc);
}

///////////////////////////////////////////////////////////////

int main(int argc, char **argv) {
	//int red_bits, green_bits, blue_bits;

  /* initialize form */
	fl_initialize(&argc, argv, "Paint", 0, 0);
	form = create_form_Paint();

	fl_set_form_position(form->Paint, 310, 0);

	/* reveal the form */
	fl_show_form(form->Paint,  	/* form */
				 FL_PLACE_POSITION,   /* pos & size flags */
				 FL_TRANSIENT,   /* border flags */
				 argv[0]          /* window name */
				 );

	/* bind OpenGL pane to display */
	glxf_bind_pane(&pane, form->pane, 0, handle_event);

	if (argc == 2) {
		ReadAndDisplayImage(argv[1]);
	}
	else {
		ReadAndDisplayImage("blank.tif");
	}
	initgl();
	InitializeMenus();
	glFlush();

	/* Xforms handles events until "exit" */
	while ((fl_check_forms() != form->exit) &&
	(fl_check_forms() != form->Exit)){
		error_check(__LINE__);
	}

	return 0;
}
