#ifndef _RAD_EQN_H
#define _RAD_EQN_H

#include <time.h>
//#include "../source/environ.c"

class RadEqnSolve   // Radiosity equation solver
{
  protected:
    int step_count;         // Step count
    int max_step;           // Maximum number of steps
    double stop_criterion;  // Stopping criterion
    double convergence;     // Convergence
    double total_area;      // Total patch area
    double total_flux;      // Total environment flux
    double total_unsent;    // Total unsent exitance
    char timebuff[10];      // Calculation time string buff
    clock_t start;          // Calculation start time
    BOOL amb_flag;          // Ambient exitance flag
    Environ *penv;          // Environment pointer
    Patch3 *pmax;           // Maximum unsent flux patch ptr
    Spectra ambient;        // Ambient exitance
    Spectra irf;            // Interreflection factors

    clock_t GetElapsed() { return (clock() - start); }
    void AddEmittance();
    void CalcAmbient();
    void CalcInterReflect();
    void InitExitance();
    void UpdateUnsentStats();

  public:
    RadEqnSolve()
    {
      amb_flag = FALSE;
      max_step = 250;
      stop_criterion = 0.001;
    }

    virtual ~RadEqnSolve() { Close(); }

    BOOL AmbientFlag() { return amb_flag; }
    BOOL Calculate() { return TRUE; }
    BOOL GetStatus() { return TRUE; }

    BOOL Open( Environ * )
    {
      start = clock();
      return TRUE;
    }

    BOOL OverShootFlag() { return FALSE; }
    char *GetElapsedTime();
    double GetStopCriterion() { return stop_criterion; }
    double GetConvergence() { return convergence; }
    int GetMaxStep() { return max_step; }
    int GetStepCount() { return step_count; }
    void Close() { }
    void DisableAmbient() { amb_flag = FALSE; }
    void DisableOverShoot() { }
    void EnableAmbient() { amb_flag = TRUE; }
    void EnableOverShoot() { }
    void SetMaxStep( int max ) { max_step = max; }
    void SetStopCriterion( double s )
    { stop_criterion = s; }
    void Shade( Environ * );
};

#endif

