#ifndef _VIEW_SYS_H
#define _VIEW_SYS_H

#include "patch3.h"
#include "environ.h"

// Extents fill factor
static const double VS_FILL_FACTOR = 0.9;

class ViewSys           // Viewing system
{
  private:
    BOOL par_flag;      // Parallel projection flag
    double bpd;         // Back plane distance
    double eye;         // View distance
    double efd;         // Eye-focus distance
    double fpd;         // Front plane distance
    double scale;       // Window scale
    double tilt_angle;  // Tilt angle (degrees)
    Point3 origin;      // View space origin
    Point3 eye_posn;    // Eye position
    Point3 focus_posn;  // Focus position
    Vector3 cvu;        // Canonical view-up vector
    Vector3 vdv;        // View direction vector
    Vector3 vuv;        // View-up vector
    double ptm[4][4];   // Projective transformation matrix

  protected:
    double aspect;      // Aspect ratio

//    BOOL BackFaceCull( Patch3 *);
   // double (*GetProjMatrix())[4];
    void BuildTransform();

  public:
    ViewSys()
    {
      par_flag = FALSE;
      aspect = 1.0;
      fpd = -0.99;
      bpd = 10000.0;
      eye = -1.0;
      efd = 1.0;
      scale = 1.0;
      tilt_angle = 0.0;
      eye_posn = Point3(-1.0, 0.0, 0.0);
      focus_posn = Point3(0.0, 0.0, 0.0);
      origin = Point3(0.0, 0.0, 0.0);
      cvu = Vector3(0.0, 0.0, 1.0);
      vdv = Vector3(-1.0, 0.0, 0.0);
      vuv = Vector3(0.0, 0.0, 1.0);

      BuildTransform();         // Initialize matrix
    }
//********************
   double (*GetProjMatrix())[4];   
 BOOL BackFaceCull( Patch3 *);	
//********************   
    BOOL GetParProjFlag() { return par_flag; }
    double GetBackDist() { return bpd; }
    double GetFrontDist() { return fpd; }
    double GetTiltAngle() { return tilt_angle; }
    double GetWindowScale() { return scale; }
    double GetViewDist() { return -eye; }
    Point3 &GetEyePosn() { return eye_posn; }
    Point3 &GetFocusPosn() { return focus_posn; }
    Point3 &GetOrigin() { return origin; }
    void Dolly( double );
    void InitViewSystem( Environ * );
    void Orbit( double, double );
    void Pan( double, double );
    void Rotate( double, double );
    void SetBackDist( double b ) { bpd = b; }
    void SetEyeFocusPosn( Point3 &, Point3 & );
    void SetFrontDist( double f ) { fpd = f; }
    void SetOrigin( Point3 &o ) { origin = o; }
    void SetParProjFlag( BOOL f ) { par_flag = f; }
    void SetTiltAngle( double t ) { tilt_angle = t; }
    void SetViewDist( double e ) { eye = -e; }
    void SetViewUp( Vector3 & );
    void SetWindowScale( double s ) { scale = s; }
    void Tilt( double );
    void Zoom( double );
};

// Return projective transformation matrix pointer
inline double (*ViewSys::GetProjMatrix())[4]
{ return ptm; }

#endif

