#include "../include/cubic_t.h"

void CubicTetra::CalcFormFactors( Patch3 *pp, Instance
	 *pi, float *ff_array, WORD num_elem )
{
  int i;                // Loop index
  BOOL hidden;          // Patch visibility flag
  BOOL self;            // Self patch flag
  WORD j;               // Loop index
  WORD elem_id;         // Element identifier
  Element3 *pelem;      // Element pointer
  Instance *pinst;      // Instance pointer
  Patch3 *ppatch;       // Patch pointer
  Surface3 *psurf;      // Surface pointer

  // Clear the form factors array
  for (j = 0; j < num_elem; j++)
	 ff_array[j] = (float) 0.0;

  // Set the cubic tetrahedron view transformation matrix
  clip.SetView(pp);

  // Project environment onto each cubic tetrahedron face
  for (i = 0; i < CubicFaceNum; i++)
  {
	 // Update view transformation matrix
	 clip.UpdateView(i);

	 scan.InitBuffer();  // Reinitialize depth buffer

	 // Walk the instance list
	 elem_id = 1;
	 pinst = pi;
	 while (pinst != NULL)
	 {
		// Walk the surface list
		psurf = pinst->GetSurfPtr();
		while (psurf != NULL)
		{
		  // Walk the patch list
		  ppatch = psurf->GetPatchPtr();
		  while (ppatch != NULL)
		  {
			 // Check for self patch
			 self = (ppatch == pp) ? TRUE : FALSE;

			 // Determine patch visibility
			 hidden = clip.BackFaceCull(ppatch);

			 // Walk the element list
			 pelem = ppatch->GetElementPtr();
			 while (pelem != NULL)
			 {
				if (hidden == FALSE && self == FALSE)
				{
				  // Clip element to face view volume
				  if (clip.Clip(pelem, out, elem_id) > 0)
				  {
					 scan.Scan(out);     // Scan convert polygon
				  }
				}
				pelem = pelem->GetNext();
				elem_id++;
			 }
			 ppatch = ppatch->GetNext();
		  }
		  psurf = psurf->GetNext();
		}
		pinst = pinst->GetNext();
	 }

	 // Sum delta form factors
	 scan.SumDeltas(ff_array);
  }
}


